 /*=========================================================================*/
 /*                              INSTALL.C                                  */
 /*                                                                         */
 /*  Functions used during the installation and uninstall of iconlock.      */
 /*                                                                         */
 /*                                                                         */
 /*                                                                         */
 /*               IconLock Version 1.0 Copyright (c) 1999                   */
 /*                         Ziff Davis, Inc.							    */
 /*                       All rights reserved.							    */
 /*        First appeared in PC Magazine, US Edition, August, 1999	        */
 /*																		    */
 /*                     Programmed by Mark Sweeney                          */
 /*                                                                         */
 /*=========================================================================*/
#include "common.h"
#include "lockdll.h"


void RemoveApprovedValue( LPCSTR lpValue );
void RemoveRegistryKeys( );
void UnregisterModule( LPCSTR lpModule );



//=====================================================================
//  FUNCTION:		RemoveQuotes
//
//  PURPOSE:		Removes quotes contained in a string
//					
//  PARAMETERS:     source string 
//
//  RETURN VALUE:	modified source string
//
//  COMMENTS:		used during uninstall
//=====================================================================
void RemoveQuotes(LPSTR pSource )
{
	LPSTR pDest = pSource;
	
	while( *pSource )	
	{
		if( *pSource != 34 )
			*pDest++ = *pSource;
		pSource++;
	}

	*pDest = 0;
}

//=====================================================================
//  FUNCTION:		IsFileLocked
//
//  PURPOSE:		Checks to see if the file is locked
//					
//  PARAMETERS:     full pathname
//
//  RETURN VALUE:	locked state	
//
//  COMMENTS:		used during uninstall
//=====================================================================
BOOL IsFileLocked( LPCSTR pFile )
{
	DWORD dwERR;
	HANDLE hFile = CreateFile( pFile, GENERIC_READ ,
						        0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );
	
	if( hFile != INVALID_HANDLE_VALUE )
	{		
		CloseHandle( hFile );
		return( FALSE );
	}

	dwERR = GetLastError( );			

	return( ( dwERR == ERROR_SHARING_VIOLATION ) || ( dwERR == ERROR_LOCK_VIOLATION ) );
}


//=====================================================================
//  FUNCTION:		UninstInitialize
//
//  PURPOSE:		checks to see if the desktop is locked 
//					
//  PARAMETERS:     
//
//  RETURN VALUE:	-1 to terminate the uninstall
//
//  COMMENTS:		called by InstallShield
//=====================================================================
int EXPORTED CALLBACK UninstInitialize( HWND hDlg, HINSTANCE hInst,LONG lReserved )
{
	HKEY hKey;
	BYTE szData[MAX_PATH * 2];
	BYTE szRun[MAX_PATH * 2];
	DWORD dwType, dwSize;
	LPSTR p,pTmp;
	int bRet = 0;

	
	//Locate the path to our data file.	
	if( RegOpenKeyEx( HKEY_LOCAL_MACHINE, 
			"Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\IconLock",
			0, KEY_ALL_ACCESS, &hKey ) == ERROR_SUCCESS )

	{
		dwSize = sizeof( szData );
		
		if( RegQueryValueEx( hKey, "UninstallString", NULL, &dwType, szData, &dwSize ) == ERROR_SUCCESS )
		{
			strcpy( szRun, szData );
			p = strstr( szData,"-c" );
			if( p ) 
			{
				p += 2;
				RemoveQuotes( p );
				pTmp = strrchr( p, '\\' );
				if( pTmp )
				{
					*(pTmp+1) = 0;
					strcat( pTmp, SZ_DATAFILE );
					ReadDataFile( p );
					if( iconlock.byDesktopLocked || IsFileLocked( p ) )
					{
						ShowError( IDS_UNINSTALL_ERROR );
						bRet = -1; //terminate the uninstall
					}
				}
			}

		}
		RegCloseKey( hKey );
		//
		if( bRet != -1 )
		{
			RemoveRegistryKeys( );
			p = strstr( szData, "-c" );
			if( p ) 
				*p = 0;
			strcat( szData, "-a" ); //silent uninstall
			bRet = -1;

			if( RegOpenKeyEx( HKEY_LOCAL_MACHINE, 
				"Software\\Microsoft\\Windows\\CurrentVersion\\RunOnce", 
				0, KEY_ALL_ACCESS, &hKey ) == ERROR_SUCCESS )
			{
				RegSetValueEx(   hKey, 
								 "IconLock", 
								 0, 
								 REG_SZ, 
								 szData, 
								 lstrlen(szData) + 1); 
				RegCloseKey( hKey );
			}
			ShowError( IDS_REBOOT );
		}
		
	}
		
	return bRet;
	
}


//=====================================================================
//  FUNCTION:		UninstUnInitialize
//
//  PURPOSE:		
//					
//  PARAMETERS:     
//
//  RETURN VALUE:	
//
//  COMMENTS:		called by InstallShield
//=====================================================================
int EXPORTED CALLBACK UninstUnInitialize( HWND hDlg, HINSTANCE hInst,LONG lReserved )
{
	return 0;
}


//=====================================================================
//  FUNCTION:		RemoveRegistryKeys
//
//  PURPOSE:		clears the registry of all IconLock entries		
//					
//  PARAMETERS:     
//
//  RETURN VALUE:	
//
//  COMMENTS:		Most of this is done by InstallShield and ATL
//=====================================================================
void RemoveRegistryKeys( )
{
  	
	UnregisterModule( "LockCopy.dll" );
	UnregisterModule( "LockDlgs.dll" );
	UnregisterModule( "LockHlpr.dll" );
	
	
	RecursiveRegDeleteKey( HKEY_CLASSES_ROOT,"CLSID\\{03183603-F684-11d2-A17F-00A0C90AE44B}" );
	RecursiveRegDeleteKey( HKEY_CLASSES_ROOT,"CLSID\\{2166382D-15FE-11D3-A17F-00A0C90AE44B}" );
	RecursiveRegDeleteKey( HKEY_CLASSES_ROOT,"CLSID\\{7248D1B1-159D-11D3-A17F-00A0C90AE44B}" );
	RecursiveRegDeleteKey( HKEY_CLASSES_ROOT,"IconHlprObj.IconHlprObj" );
	RecursiveRegDeleteKey( HKEY_CLASSES_ROOT,"IconHlprObj.IconHlprObj.1" );
	RecursiveRegDeleteKey( HKEY_CLASSES_ROOT,"Interface\\{2166382C-15FE-11D3-A17F-00A0C90AE44B}" );
	RecursiveRegDeleteKey( HKEY_CLASSES_ROOT,"Interface\\{7248D1B0-159D-11D3-A17F-00A0C90AE44B}" );
	RecursiveRegDeleteKey( HKEY_CLASSES_ROOT,"Interface\\{8E136D60-F030-11D2-A17F-00A0C90AE44B}" );
	RecursiveRegDeleteKey( HKEY_CLASSES_ROOT,"LockCopy.CallBack" );
	RecursiveRegDeleteKey( HKEY_CLASSES_ROOT,"LockCopy.CallBack.1" );
	RecursiveRegDeleteKey( HKEY_CLASSES_ROOT,"LockDlgs.LockFolder" );
	RecursiveRegDeleteKey( HKEY_CLASSES_ROOT,"LockDlgs.LockFolder.1" );
	RecursiveRegDeleteKey( HKEY_CLASSES_ROOT,"TypeLib\\{21663820-15FE-11D3-A17F-00A0C90AE44B}" );
	RecursiveRegDeleteKey( HKEY_CLASSES_ROOT,"TypeLib\\{7248D1A4-159D-11D3-A17F-00A0C90AE44B}" );
	RecursiveRegDeleteKey( HKEY_CLASSES_ROOT,"TypeLib\\{8E136D61-F030-11D2-A17F-00A0C90AE44B}" );//1.1

	RecursiveRegDeleteKey( HKEY_CLASSES_ROOT , "Directory\\ShellEx\\CopyHookHandlers\\LockCopy" ); //lockcopy
    RecursiveRegDeleteKey( HKEY_LOCAL_MACHINE, "Software\\Microsoft\\Windows\\CurrentVersion\\explorer\\Browser Helper Objects\\{03183603-F684-11d2-A17F-00A0C90AE44B}" );

	RemoveApprovedValue( "{03183603-F684-11d2-A17F-00A0C90AE44B}" );//lockhlpr
	RemoveApprovedValue( "{7248D1B1-159D-11D3-A17F-00A0C90AE44B}" );//lockfolder
	RemoveApprovedValue( "{2166382D-15FE-11D3-A17F-00A0C90AE44B}" );//lockcopy

																																

}


//=====================================================================
//  FUNCTION:		RemoveApprovedValue
//
//  PURPOSE:		removes the approved shell ext key for NT	
//					
//  PARAMETERS:     value key name
//
//  RETURN VALUE:	
//
//  COMMENTS:		
//=====================================================================
void RemoveApprovedValue( LPCSTR lpValue )
{
	HKEY hSubKey;
	char szKey[]= {"Software\\Microsoft\\Windows\\CurrentVersion\\Shell Extensions\\Approved"};

	if( RegOpenKeyEx( HKEY_LOCAL_MACHINE, szKey, 0, KEY_ALL_ACCESS, &hSubKey ) == ERROR_SUCCESS )
	{
		RegDeleteValue( hSubKey, lpValue );
		RegCloseKey( hSubKey );
	}

}


//=====================================================================
//  FUNCTION:		UnregisterModule
//
//  PURPOSE:		calls the DllUnregisterServer function 
//					
//  PARAMETERS:     full path to dll extension
//
//  RETURN VALUE:	
//
//  COMMENTS:		
//=====================================================================
void UnregisterModule( LPCSTR lpModule )
{
HMODULE hModule;
DWORD dwErr = 0;
	
	// Initialize OLE libraries
	OleInitialize( NULL );
	
	hModule = LoadLibrary( lpModule );
	if( hModule )
	{
		
		typedef HRESULT (STDAPICALLTYPE *CTLREGPROC)(void) ; //see COMPOBJ.H
		CTLREGPROC DLLUnRegisterServer = (CTLREGPROC)GetProcAddress( hModule, "DllUnregisterServer" ) ;
	
		if( DLLUnRegisterServer != NULL)
		{
			DLLUnRegisterServer( );
		} else {	
			dwErr = GetLastError( );
		}		

		FreeLibrary( hModule );          
	}
	DEBUGOUT( "Unregister Server err=%x %s", dwErr, lpModule);
	OleUninitialize();
	return;	
}


//=====================================================================
//  FUNCTION:		RecursiveRegDeleteKey
//
//  PURPOSE:		removes a registry key and all subkeys
//					
//  PARAMETERS:     standard HK and key name
//
//  RETURN VALUE:	
//
//  COMMENTS:		NT requires subkeys to be deleted first	
//=====================================================================
LONG RecursiveRegDeleteKey( HKEY hStartKey, LPCSTR pKeyName )   
{
      #define MAX_KEY_LENGTH 256
	  DWORD   dwRtn, dwSubKeyLength;      
	  LPTSTR  pSubKey = NULL;
      TCHAR   szSubKey[MAX_KEY_LENGTH]; 
      HKEY    hKey;      

	  
	  // Do not allow NULL or empty key name
      if ( pKeyName && lstrlen( pKeyName ) ) 
	  {
         if( (dwRtn = RegOpenKeyEx( hStartKey, pKeyName,
				0, KEY_ENUMERATE_SUB_KEYS | DELETE, &hKey ) ) == ERROR_SUCCESS)
         {            
			while( dwRtn == ERROR_SUCCESS )            
			{
               dwSubKeyLength = MAX_KEY_LENGTH;
               dwRtn = RegEnumKeyEx( hKey,
									0,       // always index zero
									szSubKey,
									&dwSubKeyLength,
									NULL, NULL,
									NULL, NULL );               
				if( dwRtn == ERROR_NO_MORE_ITEMS )
				{                  
					dwRtn = RegDeleteKey( hStartKey, pKeyName );
					break;               
				} else 
					if( dwRtn == ERROR_SUCCESS )
						dwRtn = RecursiveRegDeleteKey( hKey, szSubKey );            
			}
            
			RegCloseKey( hKey );
            // Do not save return code because error
            // has already occurred         
			}      
		} else
			dwRtn = ERROR_BADKEY;      
	
	return dwRtn;   
}


