/*=========================================================================*/
/*                              ICONLOCK.C                                 */
/*                                                                         */
/*     Loads the LockDLL.dll file to hook ProgMan.                         */
/*                                                                         */
/*     Normally this program is launched from the Run registry entry       */
/*     however it can be run manually.                                     */
/*                                                                         */
/*     Note:  When building the Exe and DLL you will need to build the     */
/*     DLL first in order to create the LOCKDLL.LIB library.               */
/*                                                                         */
/*               IconLock Version 1.0 Copyright (c) 1999                   */
/*                         Ziff Davis, Inc.							       */
/*                       All rights reserved.							   */
/*        First appeared in PC Magazine, US Edition, August, 1999	       */
/*																		   */
/*                     Programmed by Mark Sweeney                          */
/*                                                                         */
/*=========================================================================*/
#include <Windows.h>
#include "resource.h"
#include "iconlock.h"

/****************************/
/*          Locals          */
/****************************/
HWND hWndShell;


/****************************/
/*         Prototypes       */
/****************************/
void UnLoadProgMan( );
int ShowError( WORD wID, WORD wStyle );
LRESULT CALLBACK WindowProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam );
void _cdecl DebugOut( char *format, ... );
HWND DoInit( HINSTANCE hInst );



//=====================================================================
//  FUNCTION:		WinMain
//
//  PURPOSE:		Standard Win32 entry point.
//
//  PARAMETERS:		standard		
//
//  RETURN VALUE:	n/a
//
//  COMMENTS:		Calls the LockDLL.DLL to hook ProgMan.
//                  for the entire session.  The window will be hidden
//                  under ProgMan's window.
//=====================================================================
int WINAPI WinMain( HINSTANCE hInst, HINSTANCE hPrev, LPSTR lpCmd, int nShow )
{
HWND hWnd;																
MSG msg;
HHOOK hHook;

	//only allow one instance
	if( FindWindowEx( FindWindow( "Progman", 0), 0, SZ_CLASS_NAME, 0 ) )
		return 0;

	//OLE must be up and running to remove the drop targets from the desktop
	OleInitialize( NULL );

	//create our hidden window
	hWnd = DoInit( hInst );
	if( !hWnd )
	{
		ShowError( IDS_LOADERROR, MB_OK );
		return 1;
	}


	//Call the LockDLL function to hook progman
	//Note that the returned value is the hook'ed handle from ProgMan
	//Since we hooked it from within this process space
	//we must also unhook it here.  See below
	hHook = HookProgMan( GetCurrentThreadId(), &hWndShell );

	if( !hHook )
	{
		ShowError( IDS_LOADERROR, MB_OK );
		return 1;
	}
	

	while( GetMessage( &msg, NULL, 0, 0 ) )	//must use NULL
	{
		TranslateMessage((LPMSG)&msg);
		DispatchMessage((LPMSG)&msg);
	}

	UnregisterClass( SZ_CLASS_NAME, hInst );	
	
	//private message to unload DLL
	SendMessage( hWndShell, RegisterWindowMessage( SZ_SHUTDOWN ),0,0 );	

	//remove the hook from ProgMan
	if( hHook )
		UnhookWindowsHookEx( hHook );
	
	return( TRUE );

}




//=====================================================================
//  FUNCTION:		DoInit 
//
//  PURPOSE:		
//
//  PARAMETERS:		hinstance 
//
//  RETURN VALUE:	Hwnd to hidden window.
//
//  COMMENTS:		Creates a hidden window that will stay active
//                  for the entier session.  The window will be hidden
//                  under ProgMan's window.
//=====================================================================
HWND DoInit( HINSTANCE hInst )
{
WNDCLASS wc;

	//Register as a service so we don't display in the task list -- Win9x only
	#ifndef _DEBUG
	HANDLE  hLib = LoadLibrary( "kernel32.dll" );
	FARPROC fp = GetProcAddress( hLib, "RegisterServiceProcess" );
	if( fp )
		(*fp)( NULL, TRUE );
	FreeLibrary(hLib); 
	#endif


	if( !GetClassInfo( hInst, SZ_CLASS_NAME, &wc) )
	{
	   ZeroMemory( &wc, sizeof(wc));
	   wc.lpfnWndProc    = (WNDPROC)WindowProc;
	   wc.hInstance      = hInst;
	   wc.lpszClassName  = SZ_CLASS_NAME;
   
	   if(!RegisterClass(&wc))
		  return 0;
	}

	return( CreateWindowEx(WS_EX_TOOLWINDOW,
                          SZ_CLASS_NAME,
                          "",
                          WS_CHILD,
						  0,0,0,0,
						  FindWindow( "Progman", NULL ),
                          (HMENU)5555,
                          hInst,
                          NULL ) );
}


//=====================================================================
//  FUNCTION:		ShowError
//
//  PURPOSE:		Display a MessageBox using a string resource
//
//  PARAMETERS:		wID -- the resource ID
//                  wStyle -- MessageBox style
//
//  RETURN VALUE:	n/a
//
//  COMMENTS:		
//=====================================================================
int ShowError( WORD wID, WORD wStyle )
{
char szMsg[256];
LPCSTR p;

	if( LoadString( NULL, wID, szMsg, sizeof( szMsg ) ) )
		p = szMsg;
	else
		p = "Unknown Error";

	return( MessageBox( NULL, p, SZ_APPNAME, wStyle | MB_SETFOREGROUND ) );
}


//=====================================================================
//  FUNCTION:		WindowProc
//
//  PURPOSE:		Standard windows callback for our hidden window	
//
//  PARAMETERS:		standard
//
//  RETURN VALUE:	standard
//
//  COMMENTS:		1.1 -- moved the function to display the ShutDown
//                  dialog here, into the EXE.  This solved a problem
//                  under Win95 using the Log off command.  When this
//                  function was in the LockDLL.dll Explorer would
//                  hang. 
//=====================================================================
LRESULT CALLBACK WindowProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{

	if( msg == MSG_SHUTDOWNDLG )
	{
		UINT iRet;
		HANDLE hShell32 = LoadLibrary(TEXT("shell32.dll"));
		FARPROC fp = GetProcAddress( hShell32, (LPCSTR)MAKELPARAM( 60,0 ) );
		
		if( fp )
			iRet = fp( 0 );
	}

	return( DefWindowProc( hWnd, msg, wParam, lParam ) );
}


//=====================================================================
//  FUNCTION:		DebugOut
//
//  PURPOSE:		output text to the debugging monitor
//
//  PARAMETERS:		variable
//
//  RETURN VALUE:	standard
//
//  COMMENTS:		used only if debug is defined.
//=====================================================================
void _cdecl DebugOut( char *format, ... )
{
#ifdef _DEBUG
    char szBuf[512];
    va_list argptr;

    va_start( argptr, format );
	wvsprintf( szBuf, format, argptr );
    va_end( argptr);
	OutputDebugString( szBuf );
	OutputDebugString( "\n\r" );
#endif
}


